@@ -442,6 +442,9 @@ static HRESULT WINAPI ddraw_IDirectDrawMediaStream_CreateSample(IDirectDrawMedia
     TRACE("stream %p, surface %p, rect %s, flags %#x, sample %p.\n",
             stream, surface, wine_dbgstr_rect(rect), flags, sample);
 
+    if (!surface && rect)
+        return E_INVALIDARG;
+
     EnterCriticalSection(&stream->cs);
     hr = ddrawstreamsample_create(stream, surface, rect, sample);
     LeaveCriticalSection(&stream->cs);
@@ -632,16 +635,6 @@ static HRESULT WINAPI ddraw_sink_Connect(IPin *iface, IPin *peer, const AM_MEDIA
     return E_UNEXPECTED;
 }
 
-static BOOL check_media_type(const AM_MEDIA_TYPE *mt)
-{
-    if (IsEqualGUID(&mt->majortype, &MEDIATYPE_Video)
-            && IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_RGB8)
-            && IsEqualGUID(&mt->formattype, &FORMAT_VideoInfo))
-        return TRUE;
-
-    return FALSE;
-}
-
 static HRESULT WINAPI ddraw_sink_ReceiveConnection(IPin *iface, IPin *peer, const AM_MEDIA_TYPE *mt)
 {
     struct ddraw_stream *stream = impl_from_IPin(iface);
@@ -657,7 +650,13 @@ static HRESULT WINAPI ddraw_sink_ReceiveConnection(IPin *iface, IPin *peer, cons
         return VFW_E_ALREADY_CONNECTED;
     }
 
-    if (!check_media_type(mt))
+    if (!IsEqualGUID(&mt->majortype, &MEDIATYPE_Video)
+            || (!IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_RGB8)
+                && !IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_RGB24)
+                && !IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_RGB32)
+                && !IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_RGB555)
+                && !IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_RGB565))
+            || !IsEqualGUID(&mt->formattype, &FORMAT_VideoInfo))
     {
         LeaveCriticalSection(&stream->cs);
         return VFW_E_TYPE_NOT_ACCEPTED;
@@ -788,7 +787,13 @@ static HRESULT WINAPI ddraw_sink_QueryId(IPin *iface, WCHAR **id)
 static HRESULT WINAPI ddraw_sink_QueryAccept(IPin *iface, const AM_MEDIA_TYPE *mt)
 {
     TRACE("iface %p, mt %p.\n", iface, mt);
-    return check_media_type(mt) ? S_OK : VFW_E_TYPE_NOT_ACCEPTED;
+
+    if (IsEqualGUID(&mt->majortype, &MEDIATYPE_Video)
+            && IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_RGB8)
+            && IsEqualGUID(&mt->formattype, &FORMAT_VideoInfo))
+        return S_OK;
+
+    return VFW_E_TYPE_NOT_ACCEPTED;
 }
 
 static HRESULT WINAPI ddraw_sink_EnumMediaTypes(IPin *iface, IEnumMediaTypes **enum_media_types)
@@ -1043,6 +1048,8 @@ static ULONG WINAPI ddraw_sample_Release(IDirectDrawStreamSample *iface)
         --sample->parent->sample_refs;
         LeaveCriticalSection(&sample->parent->cs);
 
+        IAMMediaStream_Release(&sample->parent->IAMMediaStream_iface);
+
         if (sample->surface)
             IDirectDrawSurface_Release(sample->surface);
         HeapFree(GetProcessHeap(), 0, sample);
@@ -1158,6 +1165,7 @@ static HRESULT ddrawstreamsample_create(struct ddraw_stream *parent, IDirectDraw
     object->IDirectDrawStreamSample_iface.lpVtbl = &DirectDrawStreamSample_Vtbl;
     object->ref = 1;
     object->parent = parent;
+    IAMMediaStream_AddRef(&parent->IAMMediaStream_iface);
     ++parent->sample_refs;
 
     if (surface)
