@@ -741,15 +741,6 @@ DWORD WINAPI GetMaximumProcessorCount(WORD group)
     return cpus;
 }
 
-/***********************************************************************
- *           GetEnabledXStateFeatures (KERNEL32.@)
- */
-DWORD64 WINAPI GetEnabledXStateFeatures(void)
-{
-    FIXME("\n");
-    return 0;
-}
-
 /***********************************************************************
  *           GetFirmwareEnvironmentVariableA     (KERNEL32.@)
  */
