@@ -1290,7 +1290,7 @@ static BOOL delete_section( const WCHAR *filename, const WCHAR *section )
             {
                 if (!(path = get_key_value( subkey, entry )))
                 {
-                    HeapFree( GetProcessHeap(), 0, path );
+                    HeapFree( GetProcessHeap(), 0, entry );
                     continue;
                 }
 
@@ -1547,8 +1547,6 @@ UINT WINAPI GetPrivateProfileIntW( LPCWSTR section, LPCWSTR entry,
 
 /***********************************************************************
  *           GetPrivateProfileIntA   (KERNEL32.@)
- *
- * FIXME: rewrite using unicode
  */
 UINT WINAPI GetPrivateProfileIntA( LPCSTR section, LPCSTR entry,
 				   INT def_val, LPCSTR filename )
@@ -1777,11 +1775,13 @@ BOOL WINAPI WritePrivateProfileSectionW( LPCWSTR section,
                     RegCloseKey( section_key );
                     if (res)
                     {
+                        HeapFree( GetProcessHeap(), 0, entry_copy );
                         SetLastError( res );
                         RegCloseKey( key );
                         return FALSE;
                     }
                 }
+                HeapFree( GetProcessHeap(), 0, entry_copy );
             }
         }
         RegCloseKey( key );
@@ -1919,6 +1919,7 @@ DWORD WINAPI GetPrivateProfileSectionNamesW( LPWSTR buffer, DWORD size,
         {
             lstrcpynW( buffer + ret, section, size - ret - 1 );
             ret = min( ret + strlenW( section ) + 1, size - 1 );
+            HeapFree( GetProcessHeap(), 0, section );
         }
 
         RegCloseKey( key );
