@@ -33,6 +33,7 @@
 #include "winnls.h"
 #include "winternl.h"
 #include "winerror.h"
+#include "ddk/wdm.h"
 
 #include "kernelbase.h"
 #include "wine/exception.h"
@@ -1169,6 +1170,23 @@ LPVOID WINAPI DECLSPEC_HOTPATCH VirtualAllocExNuma( HANDLE process, void *addr,
 }
 
 
+/***********************************************************************
+ * CPU functions
+ ***********************************************************************/
+
+
+#if defined(__i386__) || defined(__x86_64__)
+/***********************************************************************
+ *             GetEnabledXStateFeatures   (kernelbase.@)
+ */
+DWORD64 WINAPI GetEnabledXStateFeatures(void)
+{
+    TRACE( "\n" );
+    return RtlGetEnabledExtendedFeatures( ~(ULONG64)0 );
+}
+#endif
+
+
 /***********************************************************************
  * Firmware functions
  ***********************************************************************/
