@@ -27,7 +27,6 @@
 #include "objbase.h"
 #include "shlwapi.h"
 #include "wine/debug.h"
-#include "wine/unicode.h"
 #include "msipriv.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(msi);
@@ -71,7 +70,7 @@ static struct transform_desc *parse_transform_desc( const WCHAR *str )
 
     if (!(ret = msi_alloc_zero( sizeof(*ret) ))) return NULL;
 
-    q = strchrW( p, '}' );
+    q = wcschr( p, '}' );
     if (*p != '{' || !q) goto error;
 
     len = q - p + 1;
@@ -80,14 +79,14 @@ static struct transform_desc *parse_transform_desc( const WCHAR *str )
     ret->product_code_from[len] = 0;
 
     p = q + 1;
-    if (!(q = strchrW( p, ';' ))) goto error;
+    if (!(q = wcschr( p, ';' ))) goto error;
     len = q - p;
     if (!(ret->version_from = msi_alloc( (len + 1) * sizeof(WCHAR) ))) goto error;
     memcpy( ret->version_from, p, len * sizeof(WCHAR) );
     ret->version_from[len] = 0;
 
     p = q + 1;
-    q = strchrW( p, '}' );
+    q = wcschr( p, '}' );
     if (*p != '{' || !q) goto error;
 
     len = q - p + 1;
@@ -96,14 +95,14 @@ static struct transform_desc *parse_transform_desc( const WCHAR *str )
     ret->product_code_to[len] = 0;
 
     p = q + 1;
-    if (!(q = strchrW( p, ';' ))) goto error;
+    if (!(q = wcschr( p, ';' ))) goto error;
     len = q - p;
     if (!(ret->version_to = msi_alloc( (len + 1) * sizeof(WCHAR) ))) goto error;
     memcpy( ret->version_to, p, len * sizeof(WCHAR) );
     ret->version_to[len] = 0;
 
     p = q + 1;
-    q = strchrW( p, '}' );
+    q = wcschr( p, '}' );
     if (*p != '{' || !q) goto error;
 
     len = q - p + 1;
@@ -173,7 +172,7 @@ static UINT check_transform_applicable( MSIPACKAGE *package, IStorage *transform
 
     if (wanted_flags & MSITRANSFORM_VALIDATE_LANGUAGE)
     {
-        if (!template[0] || ((p = strchrW( template, ';' )) && match_language( package, atoiW( p + 1 ) )))
+        if (!template[0] || ((p = wcschr( template, ';' )) && match_language( package, wcstol( p + 1, NULL, 10 ) )))
         {
             valid_flags |= MSITRANSFORM_VALIDATE_LANGUAGE;
         }
@@ -189,7 +188,7 @@ static UINT check_transform_applicable( MSIPACKAGE *package, IStorage *transform
             msiobj_release( &si->hdr );
             return ERROR_INSTALL_PACKAGE_INVALID;
         }
-        if (!strcmpW( desc->product_code_from, product_code_installed ))
+        if (!wcscmp( desc->product_code_from, product_code_installed ))
         {
             valid_flags |= MSITRANSFORM_VALIDATE_PRODUCT;
         }
@@ -245,7 +244,7 @@ static UINT check_transform_applicable( MSIPACKAGE *package, IStorage *transform
             msiobj_release( &si->hdr );
             return ERROR_INSTALL_PACKAGE_INVALID;
         }
-        if (!strcmpW( desc->upgrade_code, upgrade_code_installed ))
+        if (!wcscmp( desc->upgrade_code, upgrade_code_installed ))
             valid_flags |= MSITRANSFORM_VALIDATE_UPGRADECODE;
         msi_free( upgrade_code_installed );
     }
@@ -275,9 +274,14 @@ static UINT apply_substorage_transform( MSIPACKAGE *package, MSIDATABASE *patch_
     {
         ret = check_transform_applicable( package, stg );
         if (ret == ERROR_SUCCESS)
-            msi_table_apply_transform( package->db, stg );
+        {
+            msi_table_apply_transform( package->db, stg, MSITRANSFORM_ERROR_VIEWTRANSFORM );
+            msi_table_apply_transform( package->db, stg, 0 );
+        }
         else
+        {
             TRACE("substorage transform %s wasn't applicable\n", debugstr_w(name));
+        }
         IStorage_Release( stg );
     }
     else
@@ -303,7 +307,7 @@ UINT msi_check_patch_applicable( MSIPACKAGE *package, MSISUMMARYINFO *si )
     guids = msi_split_string( guid_list, ';' );
     for (i = 0; guids[i] && ret != ERROR_SUCCESS; i++)
     {
-        if (!strcmpW( guids[i], product_code )) ret = ERROR_SUCCESS;
+        if (!wcscmp( guids[i], product_code )) ret = ERROR_SUCCESS;
     }
     msi_free( guids );
     msi_free( guid_list );
@@ -333,7 +337,7 @@ static UINT msi_parse_patch_summary( MSISUMMARYINFO *si, MSIPATCHINFO **patch )
         msi_free( pi );
         return ERROR_PATCH_PACKAGE_INVALID;
     }
-    if (!(p = strchrW( p + 1, '}' )))
+    if (!(p = wcschr( p + 1, '}' )))
     {
         msi_free( pi->patchcode );
         msi_free( pi );
@@ -526,7 +530,7 @@ static UINT patch_update_file_sequence( MSIDATABASE *db, const struct patch_offs
 
     LIST_FOR_EACH_ENTRY( po, &pos->files, struct patch_offset, entry )
     {
-        if (!strcmpiW( file, po->name ))
+        if (!wcsicmp( file, po->name ))
         {
             MSI_RecordSetInteger( rec, 8, seq + pos->offset_to_apply );
             r = MSI_ViewModify( view, MSIMODIFY_UPDATE, rec );
@@ -557,7 +561,7 @@ static UINT patch_update_filepatch_sequence( MSIDATABASE *db, const struct patch
 
     LIST_FOR_EACH_ENTRY( po, &pos->patches, struct patch_offset, entry )
     {
-        if (seq == po->sequence && !strcmpiW( file, po->name ))
+        if (seq == po->sequence && !wcsicmp( file, po->name ))
         {
             MSIQUERY *delete_view, *insert_view;
             MSIRECORD *rec2;
@@ -836,6 +840,38 @@ done:
     return r;
 }
 
+static DWORD is_uninstallable( MSIDATABASE *db )
+{
+    static const WCHAR query[] = {
+        'S','E','L','E','C','T',' ','`','V','a','l','u','e','`',' ','F','R','O','M',' ',
+        '`','M','s','i','P','a','t','c','h','M','e','t','a','d','a','t','a','`',' ',
+        'W','H','E','R','E',' ','`','C','o','m','p','a','n','y','`',' ','I','S',' ',
+        'N','U','L','L',' ','A','N','D',' ','`','P','r','o','p','e','r','t','y','`','=',
+        '\'','A','l','l','o','w','R','e','m','o','v','a','l','\'',0};
+    MSIQUERY *view;
+    MSIRECORD *rec;
+    DWORD ret = 0;
+
+    if (MSI_DatabaseOpenViewW( db, query, &view ) != ERROR_SUCCESS) return 0;
+    if (MSI_ViewExecute( view, 0 ) != ERROR_SUCCESS)
+    {
+        msiobj_release( &view->hdr );
+        return 0;
+    }
+
+    if (MSI_ViewFetch( view, &rec ) == ERROR_SUCCESS)
+    {
+        const WCHAR *value = MSI_RecordGetString( rec, 1 );
+        ret = wcstol( value, NULL, 10 );
+        msiobj_release( &rec->hdr );
+    }
+
+    FIXME( "check other criteria\n" );
+
+    msiobj_release( &view->hdr );
+    return ret;
+}
+
 static UINT msi_apply_patch_db( MSIPACKAGE *package, MSIDATABASE *patch_db, MSIPATCHINFO *patch )
 {
     UINT i, r = ERROR_SUCCESS;
@@ -861,7 +897,8 @@ static UINT msi_apply_patch_db( MSIPACKAGE *package, MSIDATABASE *patch_db, MSIP
     if (r != ERROR_SUCCESS)
         return r;
 
-    patch->state = MSIPATCHSTATE_APPLIED;
+    patch->uninstallable = is_uninstallable( patch_db );
+    patch->state         = MSIPATCHSTATE_APPLIED;
     list_add_tail( &package->patches, &patch->entry );
     return ERROR_SUCCESS;
 }
@@ -972,17 +1009,17 @@ UINT msi_apply_transforms( MSIPACKAGE *package )
             if (!PathIsRelativeW( xforms[i] )) transform = xforms[i];
             else
             {
-                WCHAR *p = strrchrW( package->PackagePath, '\\' );
+                WCHAR *p = wcsrchr( package->PackagePath, '\\' );
                 DWORD len = p - package->PackagePath + 1;
 
-                if (!(transform = msi_alloc( (len + strlenW( xforms[i] ) + 1) * sizeof(WCHAR)) ))
+                if (!(transform = msi_alloc( (len + lstrlenW( xforms[i] ) + 1) * sizeof(WCHAR)) ))
                 {
                     msi_free( xforms );
                     msi_free( xform_list );
                     return ERROR_OUTOFMEMORY;
                 }
                 memcpy( transform, package->PackagePath, len * sizeof(WCHAR) );
-                memcpy( transform + len, xforms[i], (strlenW( xforms[i] ) + 1) * sizeof(WCHAR) );
+                memcpy( transform + len, xforms[i], (lstrlenW( xforms[i] ) + 1) * sizeof(WCHAR) );
             }
             r = MSI_DatabaseApplyTransformW( package->db, transform, 0 );
             if (transform != xforms[i]) msi_free( transform );
@@ -1004,7 +1041,7 @@ UINT msi_apply_registered_patch( MSIPACKAGE *package, LPCWSTR patch_code )
 
     TRACE("%p, %s\n", package, debugstr_w(patch_code));
 
-    len = sizeof(patch_file) / sizeof(WCHAR);
+    len = ARRAY_SIZE( patch_file );
     r = MsiGetPatchInfoExW( patch_code, package->ProductCode, NULL, package->Context,
                             INSTALLPROPERTY_LOCALPACKAGEW, patch_file, &len );
     if (r != ERROR_SUCCESS)
