@@ -340,23 +340,7 @@ __ASM_GLOBAL_FUNC( RtlCaptureContext,
                    "movq %r15,0xf0(%rcx)\n\t"       /* context->R15 */
                    "movq (%rsp),%rax\n\t"
                    "movq %rax,0xf8(%rcx)\n\t"       /* context->Rip */
-                   "fxsave 0x100(%rcx)\n\t"         /* context->FtlSave */
-                   "movdqa %xmm0,0x1a0(%rcx)\n\t"   /* context->Xmm0 */
-                   "movdqa %xmm1,0x1b0(%rcx)\n\t"   /* context->Xmm1 */
-                   "movdqa %xmm2,0x1c0(%rcx)\n\t"   /* context->Xmm2 */
-                   "movdqa %xmm3,0x1d0(%rcx)\n\t"   /* context->Xmm3 */
-                   "movdqa %xmm4,0x1e0(%rcx)\n\t"   /* context->Xmm4 */
-                   "movdqa %xmm5,0x1f0(%rcx)\n\t"   /* context->Xmm5 */
-                   "movdqa %xmm6,0x200(%rcx)\n\t"   /* context->Xmm6 */
-                   "movdqa %xmm7,0x210(%rcx)\n\t"   /* context->Xmm7 */
-                   "movdqa %xmm8,0x220(%rcx)\n\t"   /* context->Xmm8 */
-                   "movdqa %xmm9,0x230(%rcx)\n\t"   /* context->Xmm9 */
-                   "movdqa %xmm10,0x240(%rcx)\n\t"  /* context->Xmm10 */
-                   "movdqa %xmm11,0x250(%rcx)\n\t"  /* context->Xmm11 */
-                   "movdqa %xmm12,0x260(%rcx)\n\t"  /* context->Xmm12 */
-                   "movdqa %xmm13,0x270(%rcx)\n\t"  /* context->Xmm13 */
-                   "movdqa %xmm14,0x280(%rcx)\n\t"  /* context->Xmm14 */
-                   "movdqa %xmm15,0x290(%rcx)\n\t"  /* context->Xmm15 */
+                   "fxsave 0x100(%rcx)\n\t"         /* context->FltSave */
                    "ret" );
 
 /******************************************************************************
@@ -444,6 +428,8 @@ static NTSTATUS call_stack_handlers( EXCEPTION_RECORD *rec, CONTEXT *orig_contex
     NTSTATUS status;
 
     context = *orig_context;
+    context.ContextFlags &= ~0x40; /* Clear xstate flag. */
+
     dispatch.TargetIp      = 0;
     dispatch.ContextRecord = &context;
     dispatch.HistoryTable  = &table;
@@ -1043,9 +1029,9 @@ __ASM_GLOBAL_FUNC( call_consolidate_callback,
                    __ASM_SEH(".seh_pushframe\n\t")
                    __ASM_SEH(".seh_endprologue\n\t")
 
-                   "subq $0xf8,%rsp\n\t" /* 10*16 (float regs) + 7*8 (int regs) + 32 (shadow store). */
-                   __ASM_SEH(".seh_stackalloc 0xf8\n\t")
-                   __ASM_CFI(".cfi_adjust_cfa_offset 0xf8\n\t")
+                   "subq $0x108,%rsp\n\t" /* 10*16 (float regs) + 8*8 (int regs) + 32 (shadow store) + 8 (align). */
+                   __ASM_SEH(".seh_stackalloc 0x108\n\t")
+                   __ASM_CFI(".cfi_adjust_cfa_offset 0x108\n\t")
 
                    /* Setup CFI unwind to context. */
                    "movq %rcx,0x10(%rbp)\n\t"
@@ -1072,6 +1058,9 @@ __ASM_GLOBAL_FUNC( call_consolidate_callback,
                    __ASM_CFI(".cfi_escape 0x10,0x20,0x06,0x76,0x10,0x06,0x23,0x90,0x05\n\t") /* %xmm15 */
 
                    /* Setup SEH unwind registers restore. */
+                   "movq 0xa0(%rcx),%rax\n\t" /* context->Rbp */
+                   "movq %rax,0x100(%rsp)\n\t"
+                   __ASM_SEH(".seh_savereg %rbp, 0x100\n\t")
                    "movq 0x90(%rcx),%rax\n\t" /* context->Rbx */
                    "movq %rax,0x20(%rsp)\n\t"
                    __ASM_SEH(".seh_savereg %rbx, 0x20\n\t")
@@ -1493,6 +1482,28 @@ USHORT WINAPI RtlCaptureStackBackTrace( ULONG skip, ULONG count, PVOID *buffer,
 }
 
 
+/***********************************************************************
+ *           signal_start_thread
+ */
+__ASM_GLOBAL_FUNC( signal_start_thread,
+                   "movq %rcx,%rbx\n\t"        /* context */
+                   /* clear the thread stack */
+                   "andq $~0xfff,%rcx\n\t"     /* round down to page size */
+                   "movq %gs:0x30,%rax\n\t"
+                   "movq 0x10(%rax),%rdi\n\t"  /* NtCurrentTeb()->Tib.StackLimit */
+                   "addq $0x2000,%rdi\n\t"
+                   "movq %rdi,%rsp\n\t"
+                   "subq %rdi,%rcx\n\t"
+                   "xorl %eax,%eax\n\t"
+                   "shrq $3,%rcx\n\t"
+                   "rep; stosq\n\t"
+                   /* switch to the initial context */
+                   "leaq -32(%rbx),%rsp\n\t"
+                   "movq %rbx,%rcx\n\t"
+                   "movl $1,%edx\n\t"
+                   "call " __ASM_NAME("NtContinue") )
+
+
 /**********************************************************************
  *		DbgBreakPoint   (NTDLL.@)
  */
