@@ -73,6 +73,9 @@ char **main_argv = NULL;
 char **main_envp = NULL;
 static WCHAR **main_wargv;
 
+static LCID user_lcid, system_lcid;
+static LANGID user_ui_language, system_ui_language;
+
 static char system_locale[LOCALE_NAME_MAX_LENGTH];
 static char user_locale[LOCALE_NAME_MAX_LENGTH];
 
@@ -1142,6 +1145,7 @@ NTSTATUS CDECL get_dynamic_environment( WCHAR *env, SIZE_T *size )
     }
     else status = STATUS_BUFFER_TOO_SMALL;
     *size = pos + 1;
+    free( buffer );
     return status;
 }
 
@@ -1155,9 +1159,9 @@ void CDECL get_initial_console( HANDLE *handle, HANDLE *std_in, HANDLE *std_out,
 {
     *handle = *std_in = *std_out = *std_err = 0;
     if (isatty(0) || isatty(1) || isatty(2)) *handle = (HANDLE)2; /* see kernel32/kernel_private.h */
-    if (!isatty(0)) server_fd_to_handle( 0, GENERIC_READ|SYNCHRONIZE,  OBJ_INHERIT, std_in );
-    if (!isatty(1)) server_fd_to_handle( 1, GENERIC_WRITE|SYNCHRONIZE, OBJ_INHERIT, std_out );
-    if (!isatty(2)) server_fd_to_handle( 2, GENERIC_WRITE|SYNCHRONIZE, OBJ_INHERIT, std_err );
+    if (!isatty(0)) wine_server_fd_to_handle( 0, GENERIC_READ|SYNCHRONIZE,  OBJ_INHERIT, std_in );
+    if (!isatty(1)) wine_server_fd_to_handle( 1, GENERIC_WRITE|SYNCHRONIZE, OBJ_INHERIT, std_out );
+    if (!isatty(2)) wine_server_fd_to_handle( 2, GENERIC_WRITE|SYNCHRONIZE, OBJ_INHERIT, std_err );
 }
 
 
@@ -1250,3 +1254,58 @@ void CDECL get_locales( WCHAR *sys, WCHAR *user )
     ntdll_umbstowcs( system_locale, strlen(system_locale) + 1, sys, LOCALE_NAME_MAX_LENGTH );
     ntdll_umbstowcs( user_locale, strlen(user_locale) + 1, user, LOCALE_NAME_MAX_LENGTH );
 }
+
+
+/**********************************************************************
+ *      NtQueryDefaultLocale  (NTDLL.@)
+ */
+NTSTATUS WINAPI NtQueryDefaultLocale( BOOLEAN user, LCID *lcid )
+{
+    *lcid = user ? user_lcid : system_lcid;
+    return STATUS_SUCCESS;
+}
+
+
+/**********************************************************************
+ *      NtSetDefaultLocale  (NTDLL.@)
+ */
+NTSTATUS WINAPI NtSetDefaultLocale( BOOLEAN user, LCID lcid )
+{
+    if (user) user_lcid = lcid;
+    else
+    {
+        system_lcid = lcid;
+        system_ui_language = LANGIDFROMLCID(lcid); /* there is no separate call to set it */
+    }
+    return STATUS_SUCCESS;
+}
+
+
+/**********************************************************************
+ *      NtQueryDefaultUILanguage  (NTDLL.@)
+ */
+NTSTATUS WINAPI NtQueryDefaultUILanguage( LANGID *lang )
+{
+    *lang = user_ui_language;
+    return STATUS_SUCCESS;
+}
+
+
+/**********************************************************************
+ *      NtSetDefaultUILanguage  (NTDLL.@)
+ */
+NTSTATUS WINAPI NtSetDefaultUILanguage( LANGID lang )
+{
+    user_ui_language = lang;
+    return STATUS_SUCCESS;
+}
+
+
+/**********************************************************************
+ *      NtQueryInstallUILanguage  (NTDLL.@)
+ */
+NTSTATUS WINAPI NtQueryInstallUILanguage( LANGID *lang )
+{
+    *lang = system_ui_language;
+    return STATUS_SUCCESS;
+}
