@@ -277,7 +277,9 @@ static BOOL is_builtin_path( UNICODE_STRING *path, BOOL *is_64bit )
     *is_64bit = is_win64;
     if (path->Length > sizeof(systemW) && !wcsnicmp( path->Buffer, systemW, ARRAY_SIZE(systemW) ))
     {
-        if (is_wow64 && !ntdll_get_thread_data()->wow64_redir) *is_64bit = TRUE;
+#ifndef _WIN64
+        if (NtCurrentTeb64() && NtCurrentTeb64()->TlsSlots[WOW64_TLS_FILESYSREDIR]) *is_64bit = TRUE;
+#endif
         return TRUE;
     }
     if ((is_win64 || is_wow64) && path->Length > sizeof(wow64W) &&
@@ -509,7 +511,7 @@ static int get_unix_curdir( const RTL_USER_PROCESS_PARAMETERS *params )
                          FILE_DIRECTORY_FILE | FILE_SYNCHRONOUS_IO_NONALERT );
     free( nt_name.Buffer );
     if (status) return -1;
-    server_handle_to_fd( handle, FILE_TRAVERSE, &fd, NULL );
+    wine_server_handle_to_fd( handle, FILE_TRAVERSE, &fd, NULL );
     NtClose( handle );
     return fd;
 }
@@ -546,8 +548,8 @@ static NTSTATUS spawn_process( const RTL_USER_PROCESS_PARAMETERS *params, int so
     pid_t pid;
     char **argv;
 
-    server_handle_to_fd( params->hStdInput, FILE_READ_DATA, &stdin_fd, NULL );
-    server_handle_to_fd( params->hStdOutput, FILE_WRITE_DATA, &stdout_fd, NULL );
+    wine_server_handle_to_fd( params->hStdInput, FILE_READ_DATA, &stdin_fd, NULL );
+    wine_server_handle_to_fd( params->hStdOutput, FILE_WRITE_DATA, &stdout_fd, NULL );
 
     if (!(pid = fork()))  /* child */
     {
@@ -638,7 +640,7 @@ NTSTATUS CDECL exec_process( UNICODE_STRING *path, UNICODE_STRING *cmdline, NTST
         setsockopt( socketfd[0], SOL_SOCKET, SO_PASSCRED, &enable, sizeof(enable) );
     }
 #endif
-    server_send_fd( socketfd[1] );
+    wine_server_send_fd( socketfd[1] );
     close( socketfd[1] );
 
     SERVER_START_REQ( exec_process )
@@ -699,8 +701,8 @@ static NTSTATUS fork_and_exec( UNICODE_STRING *path, int unixdir,
         fcntl( fd[1], F_SETFD, FD_CLOEXEC );
     }
 
-    server_handle_to_fd( params->hStdInput, FILE_READ_DATA, &stdin_fd, NULL );
-    server_handle_to_fd( params->hStdOutput, FILE_WRITE_DATA, &stdout_fd, NULL );
+    wine_server_handle_to_fd( params->hStdInput, FILE_READ_DATA, &stdin_fd, NULL );
+    wine_server_handle_to_fd( params->hStdOutput, FILE_WRITE_DATA, &stdout_fd, NULL );
 
     if (!(pid = fork()))  /* child */
     {
@@ -859,7 +861,7 @@ NTSTATUS WINAPI NtCreateUserProcess( HANDLE *process_handle_ptr, HANDLE *thread_
     }
 #endif
 
-    server_send_fd( socketfd[1] );
+    wine_server_send_fd( socketfd[1] );
     close( socketfd[1] );
 
     /* create the process on the server side */
@@ -1002,6 +1004,7 @@ done:
  */
 NTSTATUS WINAPI NtTerminateProcess( HANDLE handle, LONG exit_code )
 {
+    static BOOL clean_exit;
     NTSTATUS ret;
     BOOL self;
 
@@ -1013,7 +1016,12 @@ NTSTATUS WINAPI NtTerminateProcess( HANDLE handle, LONG exit_code )
         self = reply->self;
     }
     SERVER_END_REQ;
-    if (self && handle) abort_process( exit_code );
+    if (self)
+    {
+        if (!handle) clean_exit = TRUE;
+        else if (clean_exit) exit_process( exit_code );
+        else abort_process( exit_code );
+    }
     return ret;
 }
 
@@ -1574,6 +1582,28 @@ NTSTATUS WINAPI NtSetInformationProcess( HANDLE handle, PROCESSINFOCLASS class,
         }
         break;
 
+    case ProcessThreadStackAllocation:
+    {
+        void *addr = NULL;
+        SIZE_T reserve;
+        PROCESS_STACK_ALLOCATION_INFORMATION *stack = info;
+        if (size == sizeof(PROCESS_STACK_ALLOCATION_INFORMATION_EX))
+            stack = &((PROCESS_STACK_ALLOCATION_INFORMATION_EX *)info)->AllocInfo;
+        else if (size != sizeof(*stack)) return STATUS_INFO_LENGTH_MISMATCH;
+
+        reserve = stack->ReserveSize;
+        ret = NtAllocateVirtualMemory( GetCurrentProcess(), &addr, stack->ZeroBits, &reserve,
+                                       MEM_RESERVE, PAGE_READWRITE );
+        if (!ret)
+        {
+#ifdef VALGRIND_STACK_REGISTER
+            VALGRIND_STACK_REGISTER( addr, (char *)addr + reserve );
+#endif
+            stack->StackBase = addr;
+        }
+        break;
+    }
+
     default:
         FIXME( "(%p,0x%08x,%p,0x%08x) stub\n", handle, class, info, size );
         ret = STATUS_NOT_IMPLEMENTED;
@@ -1636,3 +1666,22 @@ NTSTATUS WINAPI NtResumeProcess( HANDLE handle )
     SERVER_END_REQ;
     return ret;
 }
+
+
+/***********************************************************************
+ *           __wine_make_process_system   (NTDLL.@)
+ *
+ * Mark the current process as a system process.
+ * Returns the event that is signaled when all non-system processes have exited.
+ */
+HANDLE CDECL __wine_make_process_system(void)
+{
+    HANDLE ret = 0;
+
+    SERVER_START_REQ( make_process_system )
+    {
+        if (!wine_server_call( req )) ret = wine_server_ptr_handle( reply->event );
+    }
+    SERVER_END_REQ;
+    return ret;
+}
