@@ -99,7 +99,8 @@ static void start_thread( TEB *teb )
     thread_data->pthread_id = pthread_self();
     signal_init_thread( teb );
     server_init_thread( thread_data->start, &suspend );
-    signal_start_thread( thread_data->start, thread_data->param, suspend, pRtlUserThreadStart, teb );
+    signal_start_thread( thread_data->start, thread_data->param, suspend,
+                         pRtlUserThreadStart, pLdrInitializeThunk, teb );
 }
 
 
@@ -186,7 +187,7 @@ NTSTATUS WINAPI NtCreateThreadEx( HANDLE *handle, ACCESS_MASK access, OBJECT_ATT
         free( objattr );
         return STATUS_TOO_MANY_OPENED_FILES;
     }
-    server_send_fd( request_pipe[0] );
+    wine_server_send_fd( request_pipe[0] );
 
     if (!access) access = THREAD_ALL_ACCESS;
 
@@ -519,18 +520,6 @@ NTSTATUS WINAPI NtTerminateThread( HANDLE handle, LONG exit_code )
 }
 
 
-/***********************************************************************
- *              NtContinue  (NTDLL.@)
- */
-NTSTATUS WINAPI NtContinue( CONTEXT *context, BOOLEAN alertable )
-{
-    static const LARGE_INTEGER zero_timeout;
-
-    if (alertable) server_wait( NULL, 0, SELECT_INTERRUPTIBLE | SELECT_ALERTABLE, &zero_timeout );
-    return NtSetContextThread( GetCurrentThread(), context );
-}
-
-
 /******************************************************************************
  *              NtQueueApcThread  (NTDLL.@)
  */
