@@ -17,34 +17,8 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
-#include "config.h"
 
-#include <stdio.h>
-#include <stdarg.h>
-
-#define COBJMACROS
-
-#include "windef.h"
-#include "winbase.h"
-#include "wingdi.h"
-#include "winerror.h"
-#include "objbase.h"
-
-#include "evcode.h"
-#include "strmif.h"
-#include "control.h"
-#include "vfwmsgs.h"
-/*
- *#include "amvideo.h"
- *#include "mmreg.h"
- *#include "dshow.h"
- *#include "ddraw.h"
- */
-#include "uuids.h"
-#include "qcap_main.h"
-
-#include "wine/unicode.h"
-#include "wine/debug.h"
+#include "qcap_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(qcap);
 
