@@ -130,18 +130,11 @@ HRESULT resource_init(struct wined3d_resource *resource, struct wined3d_device *
                 continue;
             }
             if ((bind_flags & WINED3D_BIND_DEPTH_STENCIL)
-                    && !(format->flags[gl_type] & (WINED3DFMT_FLAG_DEPTH | WINED3DFMT_FLAG_STENCIL)))
+                    && !(format->flags[gl_type] & WINED3DFMT_FLAG_DEPTH_STENCIL))
             {
                 WARN("Format %s cannot be used for depth/stencil buffers.\n", debug_d3dformat(format->id));
                 continue;
             }
-            if (wined3d_settings.offscreen_rendering_mode == ORM_FBO
-                    && bind_flags & (WINED3D_BIND_RENDER_TARGET | WINED3D_BIND_DEPTH_STENCIL)
-                    && !(format->flags[gl_type] & WINED3DFMT_FLAG_FBO_ATTACHABLE))
-            {
-                WARN("Render target or depth stencil is not FBO attachable.\n");
-                continue;
-            }
             if ((bind_flags & WINED3D_BIND_SHADER_RESOURCE)
                     && !(format->flags[gl_type] & WINED3DFMT_FLAG_TEXTURE))
             {
