@@ -51,7 +51,6 @@ MAKE_FUNCPTR(XcursorLibraryLoadCursor);
 
 #include "x11drv.h"
 #include "wine/server.h"
-#include "wine/library.h"
 #include "wine/unicode.h"
 #include "wine/debug.h"
 
@@ -75,8 +74,8 @@ static const UINT button_down_flags[NB_BUTTONS] =
     MOUSEEVENTF_RIGHTDOWN,
     MOUSEEVENTF_WHEEL,
     MOUSEEVENTF_WHEEL,
-    MOUSEEVENTF_XDOWN,  /* FIXME: horizontal wheel */
-    MOUSEEVENTF_XDOWN,
+    MOUSEEVENTF_HWHEEL,
+    MOUSEEVENTF_HWHEEL,
     MOUSEEVENTF_XDOWN,
     MOUSEEVENTF_XDOWN
 };
@@ -88,8 +87,8 @@ static const UINT button_up_flags[NB_BUTTONS] =
     MOUSEEVENTF_RIGHTUP,
     0,
     0,
-    MOUSEEVENTF_XUP,
-    MOUSEEVENTF_XUP,
+    0,
+    0,
     MOUSEEVENTF_XUP,
     MOUSEEVENTF_XUP
 };
@@ -101,8 +100,8 @@ static const UINT button_down_data[NB_BUTTONS] =
     0,
     WHEEL_DELTA,
     -WHEEL_DELTA,
-    XBUTTON1,
-    XBUTTON2,
+    -WHEEL_DELTA,
+    WHEEL_DELTA,
     XBUTTON1,
     XBUTTON2
 };
@@ -114,8 +113,8 @@ static const UINT button_up_data[NB_BUTTONS] =
     0,
     0,
     0,
-    XBUTTON1,
-    XBUTTON2,
+    0,
+    0,
     XBUTTON1,
     XBUTTON2
 };
@@ -151,14 +150,13 @@ MAKE_FUNCPTR(XISelectEvents);
 void X11DRV_Xcursor_Init(void)
 {
 #ifdef SONAME_LIBXCURSOR
-    xcursor_handle = wine_dlopen(SONAME_LIBXCURSOR, RTLD_NOW, NULL, 0);
-    if (!xcursor_handle)  /* wine_dlopen failed. */
+    xcursor_handle = dlopen(SONAME_LIBXCURSOR, RTLD_NOW);
+    if (!xcursor_handle)
     {
         WARN("Xcursor failed to load.  Using fallback code.\n");
         return;
     }
-#define LOAD_FUNCPTR(f) \
-        p##f = wine_dlsym(xcursor_handle, #f, NULL, 0)
+#define LOAD_FUNCPTR(f) p##f = dlsym(xcursor_handle, #f)
 
     LOAD_FUNCPTR(XcursorImageCreate);
     LOAD_FUNCPTR(XcursorImageDestroy);
@@ -550,7 +548,8 @@ BOOL clip_fullscreen_window( HWND hwnd, BOOL reset )
 {
     struct x11drv_win_data *data;
     struct x11drv_thread_data *thread_data;
-    RECT rect;
+    MONITORINFO monitor_info;
+    HMONITOR monitor;
     DWORD style;
     BOOL fullscreen;
 
@@ -561,21 +560,25 @@ BOOL clip_fullscreen_window( HWND hwnd, BOOL reset )
     /* maximized windows don't count as full screen */
     if ((style & WS_MAXIMIZE) && (style & WS_CAPTION) == WS_CAPTION) return FALSE;
     if (!(data = get_win_data( hwnd ))) return FALSE;
-    fullscreen = is_window_rect_fullscreen( &data->whole_rect );
+    fullscreen = is_window_rect_full_screen( &data->whole_rect );
     release_win_data( data );
     if (!fullscreen) return FALSE;
     if (!(thread_data = x11drv_thread_data())) return FALSE;
     if (GetTickCount() - thread_data->clip_reset < 1000) return FALSE;
     if (!reset && clipping_cursor && thread_data->clip_hwnd) return FALSE;  /* already clipping */
-    rect = get_primary_monitor_rect();
+
+    monitor = MonitorFromWindow( hwnd, MONITOR_DEFAULTTONEAREST );
+    if (!monitor) return FALSE;
+    monitor_info.cbSize = sizeof(monitor_info);
+    if (!GetMonitorInfoW( monitor, &monitor_info )) return FALSE;
     if (!grab_fullscreen)
     {
         RECT virtual_rect = get_virtual_screen_rect();
-        if (!EqualRect( &rect, &virtual_rect )) return FALSE;
-        if (root_window != DefaultRootWindow( gdi_display )) return FALSE;
+        if (!EqualRect( &monitor_info.rcMonitor, &virtual_rect )) return FALSE;
+        if (is_virtual_desktop()) return FALSE;
     }
     TRACE( "win %p clipping fullscreen\n", hwnd );
-    return grab_clipping_window( &rect );
+    return grab_clipping_window( &monitor_info.rcMonitor );
 }
 
 
@@ -1478,7 +1481,7 @@ BOOL CDECL X11DRV_SetCursorPos( INT x, INT y )
     if (!clipping_cursor &&
         XGrabPointer( data->display, root_window, False,
                       PointerMotionMask | ButtonPressMask | ButtonReleaseMask,
-                      GrabModeAsync, GrabModeAsync, root_window, None, CurrentTime ) != GrabSuccess)
+                      GrabModeAsync, GrabModeAsync, None, None, CurrentTime ) != GrabSuccess)
     {
         WARN( "refusing to warp pointer to %u, %u without exclusive grab\n", pos.x, pos.y );
         return FALSE;
@@ -1868,7 +1871,7 @@ void X11DRV_XInput2_Init(void)
 {
 #if defined(SONAME_LIBXI) && defined(HAVE_X11_EXTENSIONS_XINPUT2_H)
     int event, error;
-    void *libxi_handle = wine_dlopen( SONAME_LIBXI, RTLD_NOW, NULL, 0 );
+    void *libxi_handle = dlopen( SONAME_LIBXI, RTLD_NOW );
 
     if (!libxi_handle)
     {
@@ -1876,7 +1879,7 @@ void X11DRV_XInput2_Init(void)
         return;
     }
 #define LOAD_FUNCPTR(f) \
-    if (!(p##f = wine_dlsym( libxi_handle, #f, NULL, 0))) \
+    if (!(p##f = dlsym( libxi_handle, #f))) \
     { \
         WARN("Failed to load %s.\n", #f); \
         return; \
