@@ -45,14 +45,16 @@ struct async
     unsigned int         status;          /* current status */
     struct timeout_user *timeout;
     unsigned int         timeout_status;  /* status to report upon timeout */
-    int                  signaled;
     struct event        *event;
     async_data_t         data;            /* data for async I/O call */
     struct iosb         *iosb;            /* I/O status block */
     obj_handle_t         wait_handle;     /* pre-allocated wait handle */
-    int                  direct_result;   /* a flag if we're passing result directly from request instead of APC  */
+    unsigned int         signaled :1;
+    unsigned int         pending :1;      /* request successfully queued, but pending */
+    unsigned int         direct_result :1;/* a flag if we're passing result directly from request instead of APC  */
     struct completion   *completion;      /* completion associated with fd */
     apc_param_t          comp_key;        /* completion key associated with fd */
+    unsigned int         comp_flags;      /* completion flags */
 };
 
 static void async_dump( struct object *obj, int verbose );
@@ -78,6 +80,7 @@ static const struct object_ops async_ops =
     no_link_name,              /* link_name */
     NULL,                      /* unlink_name */
     no_open_file,              /* open_file */
+    no_kernel_obj_list,        /* get_kernel_obj_list */
     no_close_handle,           /* close_handle */
     async_destroy              /* destroy */
 };
@@ -118,6 +121,8 @@ static void async_satisfied( struct object *obj, struct wait_queue_entry *entry
         close_handle( async->thread->process, async->wait_handle );
         async->wait_handle = 0;
     }
+
+    if (async->status == STATUS_PENDING) make_wait_abandoned( entry );
 }
 
 static void async_destroy( struct object *obj )
@@ -236,9 +241,11 @@ struct async *create_async( struct fd *fd, struct thread *thread, const async_da
     async->queue         = NULL;
     async->fd            = (struct fd *)grab_object( fd );
     async->signaled      = 0;
+    async->pending       = 1;
     async->wait_handle   = 0;
     async->direct_result = 0;
     async->completion    = fd_get_completion( fd, &async->comp_key );
+    async->comp_flags    = 0;
 
     if (iosb) async->iosb = (struct iosb *)grab_object( iosb );
     else async->iosb = NULL;
@@ -256,9 +263,22 @@ struct async *create_async( struct fd *fd, struct thread *thread, const async_da
     return async;
 }
 
+void set_async_pending( struct async *async, int signal )
+{
+    if (async->status == STATUS_PENDING)
+    {
+        async->pending = 1;
+        if (signal && !async->signaled)
+        {
+            async->signaled = 1;
+            wake_up( &async->obj, 0 );
+        }
+    }
+}
+
 /* create an async associated with iosb for async-based requests
  * returned async must be passed to async_handoff */
-struct async *create_request_async( struct fd *fd, const async_data_t *data )
+struct async *create_request_async( struct fd *fd, unsigned int comp_flags, const async_data_t *data )
 {
     struct async *async;
     struct iosb *iosb;
@@ -275,16 +295,24 @@ struct async *create_request_async( struct fd *fd, const async_data_t *data )
             release_object( async );
             return NULL;
         }
+        async->pending       = 0;
         async->direct_result = 1;
+        async->comp_flags    = comp_flags;
     }
     return async;
 }
 
 /* return async object status and wait handle to client */
-obj_handle_t async_handoff( struct async *async, int success, data_size_t *result )
+obj_handle_t async_handoff( struct async *async, int success, data_size_t *result, int force_blocking )
 {
     if (!success)
     {
+        if (get_error() == STATUS_PENDING)
+        {
+            /* we don't know the result yet, so client needs to wait */
+            async->direct_result = 0;
+            return async->wait_handle;
+        }
         close_handle( async->thread->process, async->wait_handle );
         async->wait_handle = 0;
         return 0;
@@ -313,7 +341,8 @@ obj_handle_t async_handoff( struct async *async, int success, data_size_t *resul
     else
     {
         async->direct_result = 0;
-        if (!async_is_blocking( async ))
+        async->pending = 1;
+        if (!force_blocking && async->fd && is_fd_overlapped( async->fd ))
         {
             close_handle( async->thread->process, async->wait_handle);
             async->wait_handle = 0;
@@ -370,15 +399,18 @@ void async_set_result( struct object *obj, unsigned int status, apc_param_t tota
         {
             apc_call_t data;
             memset( &data, 0, sizeof(data) );
-            data.type         = APC_USER;
-            data.user.func    = async->data.apc;
-            data.user.args[0] = async->data.apc_context;
-            data.user.args[1] = async->data.iosb;
-            data.user.args[2] = 0;
+            data.type              = APC_USER;
+            data.user.user.func    = async->data.apc;
+            data.user.user.args[0] = async->data.apc_context;
+            data.user.user.args[1] = async->data.iosb;
+            data.user.user.args[2] = 0;
             thread_queue_apc( NULL, async->thread, NULL, &data );
         }
-        else if (async->data.apc_context)
+        else if (async->data.apc_context && (async->pending ||
+                 !(async->comp_flags & FILE_SKIP_COMPLETION_PORT_ON_SUCCESS)))
+        {
             add_async_completion( async, async->data.apc_context, status, total );
+        }
 
         if (async->event) set_event( async->event );
         else if (async->fd) set_fd_signaled( async->fd, 1 );
@@ -461,6 +493,7 @@ static const struct object_ops iosb_ops =
     no_link_name,             /* link_name */
     NULL,                     /* unlink_name */
     no_open_file,             /* open_file */
+    no_kernel_obj_list,       /* get_kernel_obj_list */
     no_close_handle,          /* close_handle */
     iosb_destroy              /* destroy */
 };
