@@ -273,7 +273,7 @@ static const struct object_ops named_pipe_device_file_ops =
 {
     sizeof(struct named_pipe_device_file),   /* size */
     named_pipe_device_file_dump,             /* dump */
-    no_get_type,                             /* get_type */
+    file_get_type,                           /* get_type */
     add_queue,                               /* add_queue */
     remove_queue,                            /* remove_queue */
     default_fd_signaled,                     /* signaled */
