@@ -761,8 +761,8 @@ static void output_import_thunk( const char *name, const char *table, int pos )
         output( "1:\t.long %s+%u-(1b+4)\n", table, pos );
         break;
     case CPU_ARM64:
-        output( "\tadrp x9, %s\n", table );
-        output( "\tadd x9, x9, #:lo12:%s\n", table );
+        output( "\tadrp x9, %s\n", arm64_page( table ) );
+        output( "\tadd x9, x9, #%s\n", arm64_pageoff( table ) );
         if (pos & 0xf000) output( "\tadd x9, x9, #%u\n", pos & 0xf000 );
         if (pos & 0x0f00) output( "\tadd x9, x9, #%u\n", pos & 0x0f00 );
         if (pos & 0x00f0) output( "\tadd x9, x9, #%u\n", pos & 0x00f0 );
@@ -1080,8 +1080,8 @@ static void output_delayed_import_thunks( const DLLSPEC *spec )
     case CPU_ARM64:
         output( "\tstp x29, x30, [sp,#-16]!\n" );
         output( "\tmov x29, sp\n" );
-        output( "\tadrp x9, %s\n", asm_name("__wine_spec_delay_load") );
-        output( "\tadd x9, x9, #:lo12:%s\n", asm_name("__wine_spec_delay_load") );
+        output( "\tadrp x9, %s\n", arm64_page( asm_name("__wine_spec_delay_load") ) );
+        output( "\tadd x9, x9, #%s\n", arm64_pageoff( asm_name("__wine_spec_delay_load") ) );
         output( "\tblr x9\n" );
         output( "\tmov x9, x0\n" );
         output( "\tldp x29, x30, [sp],#16\n" );
@@ -1367,17 +1367,19 @@ void output_stubs( DLLSPEC *spec )
             else output( "\t.long %u\n", odp->ordinal );
             break;
         case CPU_ARM64:
-            output( "\tadrp x0, .L__wine_spec_file_name\n" );
-            output( "\tadd x0, x0, #:lo12:.L__wine_spec_file_name\n" );
+            output( "\tadrp x0, %s\n", arm64_page(".L__wine_spec_file_name") );
+            output( "\tadd x0, x0, #%s\n", arm64_pageoff(".L__wine_spec_file_name") );
             if (exp_name)
             {
-                output( "\tadrp x1, .L%s_string\n", name );
-                output( "\tadd x1, x1, #:lo12:.L%s_string\n", name );
+                char *sym = strmake( ".L%s_string", name );
+                output( "\tadrp x1, %s\n", arm64_page( sym ) );
+                output( "\tadd x1, x1, #%s\n", arm64_pageoff( sym ) );
+                free( sym );
             }
             else
                 output( "\tmov x1, %u\n", odp->ordinal );
-            output( "\tadrp x2, %s\n", asm_name("__wine_spec_unimplemented_stub") );
-            output( "\tadd x2, x2, #:lo12:%s\n", asm_name("__wine_spec_unimplemented_stub") );
+            output( "\tadrp x2, %s\n", arm64_page( asm_name("__wine_spec_unimplemented_stub") ) );
+            output( "\tadd x2, x2, #%s\n", arm64_pageoff( asm_name("__wine_spec_unimplemented_stub") ) );
             output( "\tblr x2\n" );
             break;
         default:
@@ -1453,8 +1455,7 @@ void output_syscalls( DLLSPEC *spec )
             output_cfi( ".cfi_rel_offset %%esi,-8\n" );
             output( "\tpushl %%edi\n" );
             output_cfi( ".cfi_rel_offset %%edi,-12\n" );
-            output( "\tpushl %%fs:0x1f8\n" );  /* x86_thread_data()->syscall_frame */
-            output( "\tmovl %%esp,%%fs:0x1f8\n" );
+            output( "\tmovl %%esp,%%fs:0x1f8\n" );  /* x86_thread_data()->syscall_frame */
             output( "\tcmpl $%u,%%eax\n", count );
             output( "\tjae 3f\n" );
             if (UsePIC)
@@ -1476,8 +1477,8 @@ void output_syscalls( DLLSPEC *spec )
                 output( "\tcall *.Lsyscall_table-1b(%%eax,%%edx,4)\n" );
             else
                 output( "\tcall *.Lsyscall_table(,%%eax,4)\n" );
-            output( "\tleal -16(%%ebp),%%esp\n" );
-            output( "2:\tpopl %%fs:0x1f8\n" );
+            output( "\tleal -12(%%ebp),%%esp\n" );
+            output( "2:\tmovl $0,%%fs:0x1f8\n" );
             output( "\tpopl %%edi\n" );
             output_cfi( ".cfi_same_value %%edi\n" );
             output( "\tpopl %%esi\n" );
@@ -1497,10 +1498,8 @@ void output_syscalls( DLLSPEC *spec )
             output_cfi( ".cfi_rel_offset %%rbp,0" );
             output( "\tmovq %%rsp,%%rbp\n" );
             output_cfi( ".cfi_def_cfa_register %%rbp" );
-            output( "\tleaq -0xe8(%%rbp),%%rsp\n" );
+            output( "\tleaq -0xe0(%%rbp),%%rsp\n" );
             output( "\tmovq %%gs:0x30,%%rcx\n" );
-            output( "\tpushq 0x328(%%rcx)\n" );  /* amd64_thread_data()->syscall_frame */
-            output( "\tmovq %%rsp,0x328(%%rcx)\n" );
             output( "\tmovdqu %%xmm6,-0xe0(%%rbp)\n" );
             output( "\tmovdqu %%xmm7,-0xd0(%%rbp)\n" );
             output( "\tmovdqu %%xmm8,-0xc0(%%rbp)\n" );
@@ -1525,6 +1524,7 @@ void output_syscalls( DLLSPEC *spec )
             /* Legends of Runeterra hooks the first system call return instruction, and
              * depends on us returning to it. Adjust the return address accordingly. */
             output( "\tsubq $0xb,0x8(%%rbp)\n" );
+            output( "\tmovq %%rsp,0x328(%%rcx)\n" );  /* amd64_thread_data()->syscall_frame */
             output( "\tcmpq $%u,%%rax\n", count );
             output( "\tjae 4f\n" );
             output( "\tleaq .Lsyscall_args(%%rip),%%rcx\n" );
@@ -1544,9 +1544,8 @@ void output_syscalls( DLLSPEC *spec )
             output( "\tsubq $0x20,%%rsp\n" );
             output( "\tleaq .Lsyscall_table(%%rip),%%r10\n" );
             output( "\tcallq *(%%r10,%%rax,8)\n" );
-            output( "3:\tpushq -0xf0(%%rbp)\n" );
-            output( "\tmovq %%gs:0x30,%%rcx\n" );
-            output( "\tpopq 0x328(%%rcx)\n" );  /* prev frame */
+            output( "3:\tmovq %%gs:0x30,%%rcx\n" );
+            output( "\tmovq $0,0x328(%%rcx)\n" );
             output( "\tmovq -0x18(%%rbp),%%rdi\n" );
             output_cfi( ".cfi_same_value %%rdi" );
             output( "\tmovq -0x10(%%rbp),%%rsi\n" );
@@ -1572,11 +1571,9 @@ void output_syscalls( DLLSPEC *spec )
             output( "\tbl %s\n", asm_name("NtCurrentTeb") );
             output( "\tadd r7, r0, #0x1d8\n" );  /* arm_thread_data()->syscall_frame */
             output( "\tpop {r0-r3}\n" );
-            output( "\tldr r8, [r7]\n" );
-            output( "\tstr r8, [sp]\n" );  /* prev frame */
-            output( "\tstr sp, [r7]\n" );  /* syscall frame */
             output( "\tmrs ip, CPSR\n" );
             output( "\tstr ip, [sp, #4]\n" );
+            output( "\tstr sp, [r7]\n" );  /* syscall frame */
             output( "\tldr r5, 7f\n");
             output( "\tadd r5, pc\n");
             output( "\tldrb r5, [r5, r4]\n" );  /* syscall args */
@@ -1592,7 +1589,8 @@ void output_syscalls( DLLSPEC *spec )
             output( "\tadd r5, pc\n");
             output( "\tldr ip, [r5, r4, lsl #2]\n");  /* syscall table */
             output( "4:\tblx ip\n");
-            output( "\tstr r8, [r7]\n" );  /* prev frame */
+            output( "\tmov ip, #0\n" );
+            output( "\tstr ip, [r7]\n" );
             output( "\tsub sp, r6, #40\n" );
             output( "\tpop {r5-r11,pc}\n" );
             output( "5:\tldr r0, 9f\n" );
@@ -1605,53 +1603,50 @@ void output_syscalls( DLLSPEC *spec )
         case CPU_ARM64:
             output( "\tcmp x8, %u\n", count );
             output( "\tbcs 3f\n" );
-            output( "\tstp x29, x30, [sp,#-176]!\n" );
+            output( "\tstp x29, x30, [sp,#-160]!\n" );
             output( "\tmov x29, sp\n" );
-            output( "\tstp x27, x28, [sp, #160]\n" );
-            output( "\tstp x25, x26, [sp, #144]\n" );
-            output( "\tstp x23, x24, [sp, #128]\n" );
-            output( "\tstp x21, x22, [sp, #112]\n" );
-            output( "\tstp x19, x20, [sp, #96]\n" );
-            output( "\tstr x8, [sp, #80]\n" );
+            output( "\tstp x27, x28, [sp, #144]\n" );
+            output( "\tstp x25, x26, [sp, #128]\n" );
+            output( "\tstp x23, x24, [sp, #112]\n" );
+            output( "\tstp x21, x22, [sp, #96]\n" );
+            output( "\tstp x19, x20, [sp, #80]\n" );
             output( "\tstp x6, x7, [sp, #64]\n" );
             output( "\tstp x4, x5, [sp, #48]\n" );
             output( "\tstp x2, x3, [sp, #32]\n" );
             output( "\tstp x0, x1, [sp, #16]\n" );
+            output( "\tmov x20, x8\n" );
             output( "\tbl %s\n", asm_name("NtCurrentTeb") );
             output( "\tadd x19, x0, #0x2f8\n" );  /* arm64_thread_data()->syscall_frame */
+            output( "\tstr x29, [x19]\n" );
             output( "\tldp x0, x1, [sp, #16]\n" );
             output( "\tldp x2, x3, [sp, #32]\n" );
             output( "\tldp x4, x5, [sp, #48]\n" );
             output( "\tldp x6, x7, [sp, #64]\n" );
-            output( "\tldr x8, [sp, #80]\n" );
-            output( "\tldr x20, [x19]\n" );  /* prev frame */
-            output( "\tstr x20, [sp, #88]\n" );
-            output( "\tstr x29, [x19]\n" );  /* syscall frame */
-            output( "\tadrp x16, .Lsyscall_args\n" );
-            output( "\tadd x16, x16, #:lo12:.Lsyscall_args\n" );
-            output( "\tldrb w9, [x16, x8]\n" );
+            output( "\tadrp x16, %s\n", arm64_page(".Lsyscall_args") );
+            output( "\tadd x16, x16, #%s\n", arm64_pageoff(".Lsyscall_args") );
+            output( "\tldrb w9, [x16, x20]\n" );
             output( "\tsubs x9, x9, #64\n" );
             output( "\tbls 2f\n" );
-            output( "\tadd x11, x29, #192\n" );
+            output( "\tadd x11, x29, #176\n" );
             output( "\tsub sp, sp, x9\n" );
             output( "\ttbz x9, #3, 1f\n" );
-            output( "\tsub SP, SP, #8\n" );
+            output( "\tsub sp, sp, #8\n" );
             output( "1:\tsub x9, x9, #8\n" );
             output( "\tldr x10, [x11, x9]\n" );
             output( "\tstr x10, [sp, x9]\n" );
             output( "\tcbnz x9, 1b\n" );
-            output( "2:\tadrp x16, .Lsyscall_table\n" );
-            output( "\tadd x16, x16, #:lo12:.Lsyscall_table\n" );
-            output( "\tldr x16, [x16, x8, lsl 3]\n" );
+            output( "2:\tadrp x16, %s\n", arm64_page(".Lsyscall_table") );
+            output( "\tadd x16, x16, #%s\n", arm64_pageoff(".Lsyscall_table") );
+            output( "\tldr x16, [x16, x20, lsl 3]\n" );
             output( "\tblr x16\n" );
             output( "\tmov sp, x29\n" );
-            output( "\tstr x20, [x19]\n" );  /* prev frame */
-            output( "\tldp x19, x20, [sp, #96]\n" );
-            output( "\tldp x21, x22, [sp, #112]\n" );
-            output( "\tldp x23, x24, [sp, #128]\n" );
-            output( "\tldp x25, x26, [sp, #144]\n" );
-            output( "\tldp x27, x28, [sp, #160]\n" );
-            output( "\tldp x29, x30, [sp], #176\n" );
+            output( "\tstr xzr, [x19]\n" );
+            output( "\tldp x19, x20, [sp, #80]\n" );
+            output( "\tldp x21, x22, [sp, #96]\n" );
+            output( "\tldp x23, x24, [sp, #112]\n" );
+            output( "\tldp x25, x26, [sp, #128]\n" );
+            output( "\tldp x27, x28, [sp, #144]\n" );
+            output( "\tldp x29, x30, [sp], #160\n" );
             output( "\tret\n" );
             output( "3:\tmov x0, #0x%x\n", invalid_param & 0xffff0000 );
             output( "\tmovk x0, #0x%x\n", invalid_param & 0x0000ffff );
@@ -1698,7 +1693,7 @@ void output_syscalls( DLLSPEC *spec )
                 output( "\tmovl $%s,%%edx\n", asm_name("__wine_syscall") );
             }
             output( "\tcall *%%edx\n" );
-            output( "\tret $%u\n", get_args_size( odp ));
+            output( "\tret $%u\n", odp->type == TYPE_STDCALL ? get_args_size( odp ) : 0 );
             break;
         case CPU_x86_64:
             /* Chromium depends on syscall thunks having the same form as on
@@ -1717,7 +1712,7 @@ void output_syscalls( DLLSPEC *spec )
             output( "\t.byte 0xc3\n" );           /* ret */
             if (target_platform == PLATFORM_WINDOWS)
             {
-                output( "1:\t.byte 0xff,0x14,0x25\n" ); /* 2: callq *(__wine_syscall_dispatcher) */
+                output( "1:\t.byte 0xff,0x14,0x25\n" ); /* 1: callq *(__wine_syscall_dispatcher) */
                 output( "\t.long __wine_syscall_dispatcher\n" );
             }
             else
@@ -1741,8 +1736,8 @@ void output_syscalls( DLLSPEC *spec )
         case CPU_ARM64:
             output( "\tstp x29, x30, [sp,#-16]!\n" );
             output( "\tmov x8, #%u\n", i );
-            output( "\tadrp x16, %s\n", asm_name("__wine_syscall_dispatcher") );
-            output( "\tldr x16, [x16, #:lo12:%s]\n", asm_name("__wine_syscall_dispatcher") );
+            output( "\tadrp x16, %s\n", arm64_page( asm_name("__wine_syscall_dispatcher") ) );
+            output( "\tldr x16, [x16, #%s]\n", arm64_pageoff( asm_name("__wine_syscall_dispatcher") ) );
             output( "\tblr x16\n");
             output( "\tldp x29, x30, [sp], #16\n" );
             output( "\tret\n" );
